unit DynamicAppInterfaces;

interface

uses
  Windows, Dialogs, Contnrs, Forms, Menus, Classes, TypInfo;

type
  IDynamicForm = interface (IInterface)
    ['{EFD19E12-6C5E-4060-B9E3-35665AB5745E}']
    function GoToRecord (const recordId: string): Boolean;
    function GetMenu: TMenuItem;
    function GetModule: THandle;
  end;

  TForm = class (Forms.TForm)
  public
    constructor Create (aOwner: TComponent); override;
    destructor Destroy; override;
  end;

  TFormData = class
  private
    FtheModule: THandle;
    FtheForm: TForm;
    procedure SettheForm(const Value: TForm);
    procedure SettheModule(const Value: THandle);
  public
    property theForm: TForm read FtheForm write SettheForm;
    property theModule: THandle read FtheModule write SettheModule;
    constructor Create (aForm: TForm; aModule: THandle);
  end;

  TClassData = class
  private
    FtheModule: THandle;
    FtheFormClass: TFormClass;
    procedure SettheFormClass(const Value: TFormClass);
    procedure SettheModule(const Value: THandle);
  public
    property theFormClass: TFormClass read FtheFormClass write SettheFormClass;
    property theModule: THandle read FtheModule write SettheModule;
    constructor Create (aFormClass: TFormClass; aModule: THandle);
  end;

procedure RegisterFormClass (AFormClass: TFormClass; aModule: THandle);
procedure UnRegisterFormClass (AFormClass: TFormClass);

function FormByModule(handle: THandle): TFormData;
function ClassByModule (handle: THandle): TClassData;

var
  listFormClasses: TObjectList;
  listActiveForms: TObjectList;

implementation

function ClassDataByClass (aFormClass: TFormClass): TClassData;
var
  I: Integer;
begin
  Result := nil;
  for I := 0 to listFormClasses.Count - 1 do
  begin
    if (listFormClasses[I] as TClassData).theFormClass = aFormClass then
    begin
      Result := listFormClasses[I] as TClassData;
      Exit;
    end;
  end;
end;

function ClassByModule (handle: THandle): TClassData;
var
  I: Integer;
begin
  Result := nil;
  for I := 0 to listFormClasses.Count - 1 do
  begin
    if (listFormClasses[I] as TClassData).theModule = handle then
    begin
      Result := listFormClasses[I] as TClassData;
      Exit;
    end;
  end;
end;


procedure RegisterFormClass (AFormClass: TFormClass; aModule: THandle);
var
  cd: TClassData;
begin
//  ShowMessage ('registering class...');
  cd := TClassData.Create (AFormClass, aModule);
  listFormClasses.Add (cd);
end;

procedure UnRegisterFormClass (AFormClass: TFormClass);
var
  cd: TClassData;
begin
  cd := ClassDataByClass (aFormClass);
  if Assigned (cd) then
    listFormClasses.Remove(cd)
  else
    ShowMessage ('odd... class not found in registry: ' + aFormclass.ClassName);
end;

function FormDataByForm(aForm: TForm): TFormData;
var
  I: Integer;
begin
  Result := nil;
  for I := 0 to listActiveForms.Count - 1 do
  begin
    if (listActiveForms[I] as TFormData).theForm = aForm then
    begin
      Result := listActiveForms[I] as TFormData;
      Exit;
    end;
  end;
end;

function FormByModule(handle: THandle): TFormData;
var
  I: Integer;
begin
  Result := nil;
  for I := 0 to listActiveForms.Count - 1 do
  begin
    if (listActiveForms[I] as TFormData).theModule = handle then
    begin
      Result := listActiveForms[I] as TFormData;
      Exit;
    end;
  end;
end;


{ TForm }

constructor TForm.Create(aOwner: TComponent);
var
  Handle: THandle;
begin
  inherited;
  Handle := (self as IDynamicForm).GetModule;
  listActiveForms.Add (TFormData.Create (self, Handle));
end;

destructor TForm.Destroy;
var
  fd: TFormData;
begin
  fd := FormDataByForm (self);
  if Assigned (fd) then
    listActiveForms.Remove(fd)
  else
    ShowMessage('odd.... form not found in registry: ' + self.ClassName);

  inherited;
end;

{ TFormData }

constructor TFormData.Create(aForm: TForm; aModule: THandle);
begin
  theForm := aForm;
  theModule := aModule;
end;

procedure TFormData.SettheForm(const Value: TForm);
begin
  FtheForm := Value;
end;

procedure TFormData.SettheModule(const Value: THandle);
begin
  FtheModule := Value;
end;

{ TClassData }

constructor TClassData.Create(aFormClass: TFormClass; aModule: THandle);
begin
  theFormClass := aFormClass;
  theModule := aModule;
end;

procedure TClassData.SettheFormClass(const Value: TFormClass);
begin
  FtheFormClass := Value;
end;

procedure TClassData.SettheModule(const Value: THandle);
begin
  FtheModule := Value;
end;

initialization
  listFormClasses := TObjectList.Create (True);
  listActiveForms := TObjectList.Create (False); // not owned

finalization
  // gets called when finalizing the packages using it!!!
  // listFormClasses.Free;
  // listActiveForms.Free;
end.

