unit DynamicAppMainForm;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, FileCtrl, ComCtrls, Menus;

type
  TForm12 = class(TForm)
    PageControl1: TPageControl;
    MainMenu1: TMainMenu;
    Main1: TMenuItem;
    Dothis1: TMenuItem;
    dothat1: TMenuItem;
    TabSheet1: TTabSheet;
    TabSheet2: TTabSheet;
    FileListBox1: TFileListBox;
    Label1: TLabel;
    ListClasses: TListBox;
    Label2: TLabel;
    ListForms: TListBox;
    Label3: TLabel;
    btnRemovePackage: TButton;
    Button1: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FileListBox1Click(Sender: TObject);
    procedure ListClassesClick(Sender: TObject);
    procedure btnRemovePackageClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Button1Click(Sender: TObject);
  private
    HandlesPackages: TList;
    procedure LoadDynaPackage(PackageName: string);
  public
    procedure RefreshClassList;
    procedure RefreshFormList;
    { Public declarations }
  end;

var
  Form12: TForm12;

implementation

uses
  // add uses here as the form is a standard one (or split the core unit in two)!
  DynamicAppInterfaces;

{$R *.dfm}

procedure TForm12.ListClassesClick(Sender: TObject);
var
  strName: string;
  aFormClass: tFormClass;
  aForm: Forms.TForm;
  menuItem: TMenuItem;
  tabSheet: TTabSheet;
begin
  strName := ListClasses.Items [ListClasses.ItemIndex];
  aFormClass := TFormClass (ListClasses.Items.Objects [
    ListClasses.ItemIndex]);
  // ShowMessage ('Creating... ' + aFormClass.ClassName);
  aForm := aFormClass.Create(Application);

  tabSheet := TTabSheet.Create(self);
  tabSheet.PageControl := PageControl1;
  tabSheet.Caption := aForm.Caption;

  aForm.BorderStyle := bsNone;
  aForm.Parent := tabSheet;
  aForm.Align := alClient;
  tabSheet.Visible := True;
  PageControl1.ActivePage := tabSheet;

  menuItem := (aForm as IDynamicForm).GetMenu;
  // ShowMessage ('adding menu... ' + menuItem.Caption);
  MainMenu1.Items.Add(menuitem);
  aForm.Show;

  RefreshFormList;
end;

procedure TForm12.LoadDynaPackage(PackageName: string);
var
  Handle: HModule;
begin
  // try to load the package
  Handle := LoadPackage (PackageName);
  if Handle > 0 then
    // add to the list for later removal
    HandlesPackages.Add (Pointer(Handle))
  else
    ShowMessage ('Package ' + PackageName + ' not found');
end;

procedure TForm12.RefreshClassList;
var
  I: Integer;
  cd: TClassData;
begin
  ListClasses.Clear;
  for I := 0 to ListFormClasses.Count - 1 do
  begin
    cd := ListFormClasses [I] as TClassData;
    ListClasses.Items.AddObject (cd.theFormClass.ClassName + '|' +
      IntToStr (cd.theModule), TObject(cd.theFormClass));
  end;
end;

procedure TForm12.RefreshFormList;
var
  I: Integer;
  fd: TFormData;
begin
  ListForms.Clear;
  for I := 0 to ListFormClasses.Count - 1 do
  begin
    fd := listActiveForms [I] as TFormData;
    ListForms.Items.Add (fd.theForm.Caption + '|' +
      IntToStr (fd.theModule));
  end;
end;

procedure TForm12.btnRemovePackageClick(Sender: TObject);
var
  strName: string;
  modHandle: THandle;
  fd: TFormData;
  cd: TClassData;
begin
  strName := FileListBox1.Items [FileListBox1.ItemIndex];

  modHandle := GetModuleHandle (PChar(strName));

  // find and free live objects of the package
  while Assigned (DynamicAppInterfaces.FormByModule(modHandle)) do
  begin
    fd := DynamicAppInterfaces.FormByModule(modHandle);
    //Mainmenu1.Items.Remove(
    // (fd.theForm as IDynamicForm).GetMenu.Free;

    // remove the tab hosting the form
    (fd.theForm.Parent as TTabSheet).TabVisible := False;
    // now delete the form
    fd.theForm.Free;
    // which is not there any more...
    listActiveForms.Remove(fd);
  end;

  // find and free classes of the package
  while Assigned (DynamicAppInterfaces.ClassByModule(modHandle)) do
  begin
    cd := DynamicAppInterfaces.ClassByModule(modHandle);
    listFormClasses.Remove(cd);
  end;

  // now unload...
  UnloadPackage (modHandle);
  HandlesPackages.Remove(Pointer(modHandle));

  RefreshClassList;
  RefreshFormList;
end;

procedure TForm12.Button1Click(Sender: TObject);
begin
  FileListBox1.Refresh;
end;

procedure TForm12.FileListBox1Click(Sender: TObject);
var
  strName: string;
begin
  strName := FileListBox1.Items [FileListBox1.ItemIndex];
  // ShowMessage ('Loading... ' + strname);
  LoadDynaPackage (strname);
  RefreshClassList;
end;

procedure TForm12.FormCreate(Sender: TObject);
begin
  HandlesPackages := TList.Create;
end;

procedure TForm12.FormDestroy(Sender: TObject);
var
  I: Integer;
begin
  // unlaod all packages
  for I := 0 to HandlesPackages.Count - 1 do
    UnloadPackage (Cardinal (HandlesPackages [I]));
  // free the lists
  HandlesPackages.Free;
end;

end.
